@@ -1,4 +1,6 @@
+from __future__ import unicode_literals
 from moto.autoscaling import autoscaling_backend
+from moto.cloudwatch import cloudwatch_backend
 from moto.dynamodb import dynamodb_backend
 from moto.dynamodb2 import dynamodb_backend2
 from moto.ec2 import ec2_backend
@@ -13,6 +15,7 @@ from moto.route53 import route53_backend
 
 BACKENDS = {
     'autoscaling': autoscaling_backend,
+    'cloudwatch': cloudwatch_backend,
     'dynamodb': dynamodb_backend,
     'dynamodb2': dynamodb_backend2,
     'ec2': ec2_backend,
@@ -25,3 +28,10 @@ BACKENDS = {
     'sts': sts_backend,
     'route53': route53_backend
 }
+
+
+def get_model(name):
+    for backend in BACKENDS.values():
+        models = getattr(backend.__class__, '__models__', {})
+        if name in models:
+            return list(getattr(backend, models[name])())
