@@ -1,14 +1,51 @@
+from __future__ import unicode_literals
 import datetime
 import json
+import re
 
-from urlparse import parse_qs, urlparse
+import six
+from six.moves.urllib.parse import parse_qs, urlparse
 
 from werkzeug.exceptions import HTTPException
 from moto.core.utils import camelcase_to_underscores, method_names_from_class
 
 
+def _decode_dict(d):
+    decoded = {}
+    for key, value in d.items():
+        if isinstance(key, six.binary_type):
+            newkey = key.decode("utf-8")
+        elif isinstance(key, (list, tuple)):
+            newkey = []
+            for k in key:
+                if isinstance(k, six.binary_type):
+                    newkey.append(k.decode('utf-8'))
+                else:
+                    newkey.append(k)
+        else:
+            newkey = key
+
+        if isinstance(value, six.binary_type):
+            newvalue = value.decode("utf-8")
+        elif isinstance(value, (list, tuple)):
+            newvalue = []
+            for v in value:
+                if isinstance(v, six.binary_type):
+                    newvalue.append(v.decode('utf-8'))
+                else:
+                    newvalue.append(v)
+        else:
+            newvalue = value
+
+        decoded[newkey] = newvalue
+    return decoded
+
+
 class BaseResponse(object):
 
+    default_region = 'us-east-1'
+    region_regex = r'\.(.+?)\.amazonaws\.com'
+
     def dispatch(self, request, full_url, headers):
         querystring = {}
 
@@ -24,20 +61,27 @@ class BaseResponse(object):
             self.body = request.data
 
             querystring = {}
-            for key, value in request.form.iteritems():
+            for key, value in request.form.items():
                 querystring[key] = [value, ]
 
         if not querystring:
-            querystring.update(parse_qs(urlparse(full_url).query))
+            querystring.update(parse_qs(urlparse(full_url).query, keep_blank_values=True))
         if not querystring:
-            querystring.update(parse_qs(self.body))
+            querystring.update(parse_qs(self.body, keep_blank_values=True))
         if not querystring:
             querystring.update(headers)
 
+        querystring = _decode_dict(querystring)
+
         self.uri = full_url
         self.path = urlparse(full_url).path
         self.querystring = querystring
         self.method = request.method
+        region = re.search(self.region_regex, full_url)
+        if region:
+            self.region = region.group(1)
+        else:
+            self.region = self.default_region
 
         self.headers = dict(request.headers)
         self.response_headers = headers
@@ -54,7 +98,7 @@ class BaseResponse(object):
                 response = method()
             except HTTPException as http_error:
                 response = http_error.description, dict(status=http_error.code)
-            if isinstance(response, basestring):
+            if isinstance(response, six.string_types):
                 return 200, headers, response
             else:
                 body, new_headers = response
@@ -66,6 +110,22 @@ class BaseResponse(object):
     def _get_param(self, param_name):
         return self.querystring.get(param_name, [None])[0]
 
+    def _get_multi_param(self, param_prefix):
+        if param_prefix.endswith("."):
+            prefix = param_prefix
+        else:
+            prefix = param_prefix + "."
+        values = []
+        index = 1
+        while True:
+            try:
+                values.append(self.querystring[prefix + str(index)][0])
+            except KeyError:
+                break
+            else:
+                index += 1
+        return values
+
 
 def metadata_response(request, full_url, headers):
     """
@@ -101,4 +161,6 @@ def metadata_response(request, full_url, headers):
         result = 'default-role'
     elif path == 'iam/security-credentials/default-role':
         result = json.dumps(credentials)
+    else:
+        raise NotImplementedError("The {0} metadata path has not been implemented".format(path))
     return 200, headers, result
