@@ -1,12 +1,13 @@
+from __future__ import unicode_literals
 from collections import defaultdict
 import datetime
 import json
 
 try:
-        from collections import OrderedDict
+    from collections import OrderedDict
 except ImportError:
-        # python 2.6 or earlier, use backport
-        from ordereddict import OrderedDict
+    # python 2.6 or earlier, use backport
+    from ordereddict import OrderedDict
 
 
 from moto.core import BaseBackend
@@ -30,8 +31,8 @@ class DynamoType(object):
     """
 
     def __init__(self, type_as_dict):
-        self.type = type_as_dict.keys()[0]
-        self.value = type_as_dict.values()[0]
+        self.type = list(type_as_dict)[0]
+        self.value = list(type_as_dict.values())[0]
 
     def __hash__(self):
         return hash((self.type, self.value))
@@ -68,6 +69,7 @@ class DynamoType(object):
         comparison_func = get_comparison_func(range_comparison)
         return comparison_func(self.value, *range_values)
 
+
 class Item(object):
     def __init__(self, hash_key, hash_key_type, range_key, range_key_type, attrs):
         self.hash_key = hash_key
@@ -76,7 +78,7 @@ class Item(object):
         self.range_key_type = range_key_type
 
         self.attrs = {}
-        for key, value in attrs.iteritems():
+        for key, value in attrs.items():
             self.attrs[key] = DynamoType(value)
 
     def __repr__(self):
@@ -84,7 +86,7 @@ class Item(object):
 
     def to_json(self):
         attributes = {}
-        for attribute_key, attribute in self.attrs.iteritems():
+        for attribute_key, attribute in self.attrs.items():
             attributes[attribute_key] = attribute.value
 
         return {
@@ -94,7 +96,7 @@ class Item(object):
     def describe_attrs(self, attributes):
         if attributes:
             included = {}
-            for key, value in self.attrs.iteritems():
+            for key, value in self.attrs.items():
                 if key in attributes:
                     included[key] = value
         else:
@@ -103,9 +105,10 @@ class Item(object):
             "Item": included
         }
 
+
 class Table(object):
 
-    def __init__(self, table_name, schema=None, attr = None, throughput=None, indexes=None):
+    def __init__(self, table_name, schema=None, attr=None, throughput=None, indexes=None):
         self.name = table_name
         self.attr = attr
         self.schema = schema
@@ -121,39 +124,39 @@ class Table(object):
                 self.range_key_attr = elem["AttributeName"]
                 self.range_key_type = elem["KeyType"]
         if throughput is None:
-             self.throughput = {u'WriteCapacityUnits': 10, u'ReadCapacityUnits': 10}
+            self.throughput = {'WriteCapacityUnits': 10, 'ReadCapacityUnits': 10}
         else:
             self.throughput = throughput
         self.throughput["NumberOfDecreasesToday"] = 0
         self.indexes = indexes
         self.created_at = datetime.datetime.now()
         self.items = defaultdict(dict)
-        
+
     @property
     def describe(self):
         results = {
-        'Table': {
-            'AttributeDefinitions': self.attr,
-            'ProvisionedThroughput': self.throughput, 
-            'TableSizeBytes': 0, 
-            'TableName': self.name, 
-            'TableStatus': 'ACTIVE', 
-            'KeySchema': self.schema, 
-            'ItemCount': len(self), 
-            'CreationDateTime': unix_time(self.created_at)
+            'Table': {
+                'AttributeDefinitions': self.attr,
+                'ProvisionedThroughput': self.throughput,
+                'TableSizeBytes': 0,
+                'TableName': self.name,
+                'TableStatus': 'ACTIVE',
+                'KeySchema': self.schema,
+                'ItemCount': len(self),
+                'CreationDateTime': unix_time(self.created_at),
             }
         }
         return results
-    
+
     def __len__(self):
         count = 0
-        for key, value in self.items.iteritems():
+        for key, value in self.items.items():
             if self.has_range_key:
                 count += len(value)
             else:
                 count += 1
         return count
-    
+
     def put_item(self, item_attrs):
         hash_value = DynamoType(item_attrs.get(self.hash_key_attr))
         if self.has_range_key:
@@ -168,14 +171,17 @@ class Table(object):
         else:
             self.items[hash_value] = item
         return item
-    
+
     def __nonzero__(self):
         return True
-    
+
+    def __bool__(self):
+        return self.__nonzero__()
+
     @property
     def has_range_key(self):
         return self.range_key_attr is not None
-    
+
     def get_item(self, hash_key, range_key):
         if self.has_range_key and not range_key:
             raise ValueError("Table has a range key, but no range key was passed into get_item")
@@ -186,7 +192,7 @@ class Table(object):
                 return self.items[hash_key]
         except KeyError:
             return None
-        
+
     def delete_item(self, hash_key, range_key):
         try:
             if range_key:
@@ -195,12 +201,12 @@ class Table(object):
                 return self.items.pop(hash_key)
         except KeyError:
             return None
-        
+
     def query(self, hash_key, range_comparison, range_objs):
         results = []
         last_page = True  # Once pagination is implemented, change this
 
-        possible_results =  [ item for item in list(self.all_items()) if item.hash_key == hash_key] 
+        possible_results = [item for item in list(self.all_items()) if item.hash_key == hash_key]
         if range_comparison:
             for result in possible_results:
                 if result.range_key.compare(range_comparison, range_objs):
@@ -219,7 +225,7 @@ class Table(object):
                     yield item
             else:
                 yield hash_set
-                
+
     def scan(self, filters):
         results = []
         scanned_count = 0
@@ -228,7 +234,7 @@ class Table(object):
         for result in self.all_items():
             scanned_count += 1
             passes_all_conditions = True
-            for attribute_name, (comparison_operator, comparison_objs) in filters.iteritems():
+            for attribute_name, (comparison_operator, comparison_objs) in filters.items():
                 attribute = result.attrs.get(attribute_name)
 
                 if attribute:
@@ -247,7 +253,7 @@ class Table(object):
             if passes_all_conditions:
                 results.append(result)
         return results, scanned_count, last_page
-    
+
 
 class DynamoDBBackend(BaseBackend):
 
@@ -272,26 +278,26 @@ class DynamoDBBackend(BaseBackend):
         if not table:
             return None
         return table.put_item(item_attrs)
-    
+
     def get_table_keys_name(self, table_name):
         table = self.tables.get(table_name)
         if not table:
             return None, None
         else:
             return table.hash_key_attr, table.range_key_attr
-         
+
     def get_keys_value(self, table, keys):
-        if not table.hash_key_attr in keys or (table.has_range_key and not table.range_key_attr in keys):
-            raise ValueError("Table has a range key, but no range key was passed into get_item")        
-        hash_key = DynamoType(keys[table.hash_key_attr])    
+        if table.hash_key_attr not in keys or (table.has_range_key and table.range_key_attr not in keys):
+            raise ValueError("Table has a range key, but no range key was passed into get_item")
+        hash_key = DynamoType(keys[table.hash_key_attr])
         range_key = DynamoType(keys[table.range_key_attr]) if table.has_range_key else None
-        return hash_key,range_key
+        return hash_key, range_key
 
     def get_item(self, table_name, keys):
         table = self.tables.get(table_name)
         if not table:
             return None
-        hash_key,range_key = self.get_keys_value(table,keys)
+        hash_key, range_key = self.get_keys_value(table, keys)
         return table.get_item(hash_key, range_key)
 
     def query(self, table_name, hash_key_dict, range_comparison, range_value_dicts):
@@ -303,19 +309,19 @@ class DynamoDBBackend(BaseBackend):
         range_values = [DynamoType(range_value) for range_value in range_value_dicts]
 
         return table.query(hash_key, range_comparison, range_values)
-    
+
     def scan(self, table_name, filters):
         table = self.tables.get(table_name)
         if not table:
             return None, None, None
 
         scan_filters = {}
-        for key, (comparison_operator, comparison_values) in filters.iteritems():
+        for key, (comparison_operator, comparison_values) in filters.items():
             dynamo_types = [DynamoType(value) for value in comparison_values]
             scan_filters[key] = (comparison_operator, dynamo_types)
 
         return table.scan(scan_filters)
-    
+
     def delete_item(self, table_name, keys):
         table = self.tables.get(table_name)
         if not table:
