@@ -1,7 +1,8 @@
+from __future__ import unicode_literals
 from jinja2 import Template
 
 from moto.core.responses import BaseResponse
-from moto.ec2.models import ec2_backend
+from moto.ec2.utils import filters_from_querystring
 
 
 def process_rules_from_querystring(querystring):
@@ -18,15 +19,14 @@ def process_rules_from_querystring(querystring):
     from_port = querystring.get('IpPermissions.1.FromPort')[0]
     to_port = querystring.get('IpPermissions.1.ToPort')[0]
     ip_ranges = []
-    for key, value in querystring.iteritems():
+    for key, value in querystring.items():
         if 'IpPermissions.1.IpRanges' in key:
             ip_ranges.append(value[0])
 
-
     source_groups = []
     source_group_ids = []
 
-    for key, value in querystring.iteritems():
+    for key, value in querystring.items():
         if 'IpPermissions.1.Groups.1.GroupId' in key:
             source_group_ids.append(value[0])
         elif 'IpPermissions.1.Groups' in key:
@@ -40,17 +40,14 @@ class SecurityGroups(BaseResponse):
         raise NotImplementedError('SecurityGroups.authorize_security_group_egress is not yet implemented')
 
     def authorize_security_group_ingress(self):
-        ec2_backend.authorize_security_group_ingress(*process_rules_from_querystring(self.querystring))
+        self.ec2_backend.authorize_security_group_ingress(*process_rules_from_querystring(self.querystring))
         return AUTHORIZE_SECURITY_GROUP_INGRESS_REPONSE
 
     def create_security_group(self):
         name = self.querystring.get('GroupName')[0]
-        description = self.querystring.get('GroupDescription')[0]
+        description = self.querystring.get('GroupDescription', [None])[0]
         vpc_id = self.querystring.get("VpcId", [None])[0]
-        group = ec2_backend.create_security_group(name, description, vpc_id=vpc_id)
-        if not group:
-            # There was an exisitng group
-            return "There was an existing security group with name {0}".format(name), dict(status=409)
+        group = self.ec2_backend.create_security_group(name, description, vpc_id=vpc_id)
         template = Template(CREATE_SECURITY_GROUP_RESPONSE)
         return template.render(group=group)
 
@@ -61,18 +58,23 @@ class SecurityGroups(BaseResponse):
         sg_id = self.querystring.get('GroupId')
 
         if name:
-            group = ec2_backend.delete_security_group(name[0])
+            self.ec2_backend.delete_security_group(name[0])
         elif sg_id:
-            group = ec2_backend.delete_security_group(group_id=sg_id[0])
+            self.ec2_backend.delete_security_group(group_id=sg_id[0])
 
-        # needs name or group now
-        if not group:
-            # There was no such group
-            return "There was no security group with name {0}".format(name), dict(status=404)
         return DELETE_GROUP_RESPONSE
 
     def describe_security_groups(self):
-        groups = ec2_backend.describe_security_groups()
+        groupnames = self._get_multi_param("GroupName")
+        group_ids = self._get_multi_param("GroupId")
+        filters = filters_from_querystring(self.querystring)
+
+        groups = self.ec2_backend.describe_security_groups(
+            group_ids=group_ids,
+            groupnames=groupnames,
+            filters=filters
+        )
+
         template = Template(DESCRIBE_SECURITY_GROUPS_RESPONSE)
         return template.render(groups=groups)
 
@@ -80,9 +82,7 @@ class SecurityGroups(BaseResponse):
         raise NotImplementedError('SecurityGroups.revoke_security_group_egress is not yet implemented')
 
     def revoke_security_group_ingress(self):
-        success = ec2_backend.revoke_security_group_ingress(*process_rules_from_querystring(self.querystring))
-        if not success:
-            return "Could not find a matching ingress rule", dict(status=404)
+        self.ec2_backend.revoke_security_group_ingress(*process_rules_from_querystring(self.querystring))
         return REVOKE_SECURITY_GROUP_INGRESS_REPONSE
 
 
@@ -106,7 +106,9 @@ DESCRIBE_SECURITY_GROUPS_RESPONSE = """<DescribeSecurityGroupsResponse xmlns="ht
              <groupId>{{ group.id }}</groupId>
              <groupName>{{ group.name }}</groupName>
              <groupDescription>{{ group.description }}</groupDescription>
-             <vpcId>{{ group.vpc_id or ""}}</vpcId>
+             {% if group.vpc_id %}
+             <vpcId>{{ group.vpc_id }}</vpcId>
+             {% endif %}
              <ipPermissions>
                {% for rule in group.ingress_rules %}
                     <item>
