@@ -1,28 +1,27 @@
+from __future__ import unicode_literals
 from jinja2 import Template
 
 from moto.core.responses import BaseResponse
-from moto.ec2.models import ec2_backend
+from moto.ec2.utils import filters_from_querystring
 
 
 class Subnets(BaseResponse):
     def create_subnet(self):
         vpc_id = self.querystring.get('VpcId')[0]
         cidr_block = self.querystring.get('CidrBlock')[0]
-        subnet = ec2_backend.create_subnet(vpc_id, cidr_block)
+        subnet = self.ec2_backend.create_subnet(vpc_id, cidr_block)
         template = Template(CREATE_SUBNET_RESPONSE)
         return template.render(subnet=subnet)
 
     def delete_subnet(self):
         subnet_id = self.querystring.get('SubnetId')[0]
-        subnet = ec2_backend.delete_subnet(subnet_id)
-        if subnet:
-            template = Template(DELETE_SUBNET_RESPONSE)
-            return template.render(subnet=subnet)
-        else:
-            return "", dict(status=404)
+        subnet = self.ec2_backend.delete_subnet(subnet_id)
+        template = Template(DELETE_SUBNET_RESPONSE)
+        return template.render(subnet=subnet)
 
     def describe_subnets(self):
-        subnets = ec2_backend.get_all_subnets()
+        filters = filters_from_querystring(self.querystring)
+        subnets = self.ec2_backend.get_all_subnets(filters)
         template = Template(DESCRIBE_SUBNETS_RESPONSE)
         return template.render(subnets=subnets)
 
@@ -37,7 +36,16 @@ CREATE_SUBNET_RESPONSE = """
     <cidrBlock>{{ subnet.cidr_block }}</cidrBlock>
     <availableIpAddressCount>251</availableIpAddressCount>
     <availabilityZone>us-east-1a</availabilityZone>
-    <tagSet/>
+    <tagSet>
+      {% for tag in subnet.get_tags() %}
+        <item>
+          <resourceId>{{ tag.resource_id }}</resourceId>
+          <resourceType>{{ tag.resource_type }}</resourceType>
+          <key>{{ tag.key }}</key>
+          <value>{{ tag.value }}</value>
+        </item>
+      {% endfor %}
+    </tagSet>
   </subnet>
 </CreateSubnetResponse>"""
 
@@ -59,7 +67,16 @@ DESCRIBE_SUBNETS_RESPONSE = """
         <cidrBlock>{{ subnet.cidr_block }}</cidrBlock>
         <availableIpAddressCount>251</availableIpAddressCount>
         <availabilityZone>us-east-1a</availabilityZone>
-        <tagSet/>
+        <tagSet>
+          {% for tag in subnet.get_tags() %}
+            <item>
+              <resourceId>{{ tag.resource_id }}</resourceId>
+              <resourceType>{{ tag.resource_type }}</resourceType>
+              <key>{{ tag.key }}</key>
+              <value>{{ tag.value }}</value>
+            </item>
+          {% endfor %}
+        </tagSet>
       </item>
     {% endfor %}
   </subnetSet>
