@@ -1,5 +1,6 @@
+from __future__ import unicode_literals
 from jinja2 import Template
-from urlparse import parse_qs, urlparse
+from six.moves.urllib.parse import parse_qs, urlparse
 from .models import route53_backend
 import xmltodict
 import dicttoxml
@@ -66,8 +67,11 @@ def rrset_response(request, full_url, headers):
         template = Template(LIST_RRSET_REPONSE)
         rrset_list = []
         for key, value in the_zone.rrsets.items():
-            if 'type' not in querystring or querystring["type"][0] == value["Type"]:
-                rrset_list.append(dicttoxml.dicttoxml({"ResourceRecordSet": value}, root=False))
+            if 'type' in querystring and querystring["type"][0] != value["Type"]:
+                continue
+            if 'name' in querystring and querystring["name"][0] != value["Name"]:
+                continue
+            rrset_list.append(dicttoxml.dicttoxml({"ResourceRecordSet": value}, root=False))
 
         return 200, headers, template.render(rrsets=rrset_list)
 
@@ -82,8 +86,9 @@ LIST_RRSET_REPONSE = """<ListResourceRecordSetsResponse xmlns="https://route53.a
 
 CHANGE_RRSET_RESPONSE = """<ChangeResourceRecordSetsResponse xmlns="https://route53.amazonaws.com/doc/2012-12-12/">
    <ChangeInfo>
-      <Status>PENDING</Status>
+      <Status>INSYNC</Status>
       <SubmittedAt>2010-09-10T01:36:41.958Z</SubmittedAt>
+      <Id>/change/C2682N5HXP0BZ4</Id>
    </ChangeInfo>
 </ChangeResourceRecordSetsResponse>"""
 
@@ -116,7 +121,7 @@ CREATE_HOSTED_ZONE_RESPONSE = """<CreateHostedZoneResponse xmlns="https://route5
    </DelegationSet>
 </CreateHostedZoneResponse>"""
 
-LIST_HOSTED_ZONES_RESPONSE = """<ListHostedZonesResponse xmlns="https://route53.amazonaws.com/doc/2012-12-12/"> 
+LIST_HOSTED_ZONES_RESPONSE = """<ListHostedZonesResponse xmlns="https://route53.amazonaws.com/doc/2012-12-12/">
    <HostedZones>
       {% for zone in zones %}
       <HostedZone>
