@@ -1,9 +1,11 @@
-from urlparse import parse_qs, urlparse
+from __future__ import unicode_literals
+import six
+from six.moves.urllib.parse import parse_qs, urlparse
 import re
 
 from jinja2 import Template
 
-from .exceptions import BucketAlreadyExists
+from .exceptions import BucketAlreadyExists, MissingBucket
 from .models import s3_backend
 from .utils import bucket_name_from_url
 from xml.dom import minidom
@@ -26,8 +28,12 @@ class ResponseObject(object):
         return template.render(buckets=all_buckets)
 
     def bucket_response(self, request, full_url, headers):
-        response = self._bucket_response(request, full_url, headers)
-        if isinstance(response, basestring):
+        try:
+            response = self._bucket_response(request, full_url, headers)
+        except MissingBucket:
+            return 404, headers, ""
+
+        if isinstance(response, six.string_types):
             return 200, headers, response
         else:
             status_code, headers, response_content = response
@@ -48,7 +54,7 @@ class ResponseObject(object):
         elif method == 'GET':
             return self._bucket_response_get(bucket_name, querystring, headers)
         elif method == 'PUT':
-            return self._bucket_response_put(bucket_name, headers)
+            return self._bucket_response_put(request, bucket_name, querystring, headers)
         elif method == 'DELETE':
             return self._bucket_response_delete(bucket_name, headers)
         elif method == 'POST':
@@ -57,54 +63,97 @@ class ResponseObject(object):
             raise NotImplementedError("Method {0} has not been impelemented in the S3 backend yet".format(method))
 
     def _bucket_response_head(self, bucket_name, headers):
-        bucket = self.backend.get_bucket(bucket_name)
-        if bucket:
-            return 200, headers, ""
-        else:
+        try:
+            self.backend.get_bucket(bucket_name)
+        except MissingBucket:
             return 404, headers, ""
+        else:
+            return 200, headers, ""
 
     def _bucket_response_get(self, bucket_name, querystring, headers):
         if 'uploads' in querystring:
             for unsup in ('delimiter', 'prefix', 'max-uploads'):
                 if unsup in querystring:
                     raise NotImplementedError("Listing multipart uploads with {} has not been implemented yet.".format(unsup))
-            multiparts = list(self.backend.get_all_multiparts(bucket_name).itervalues())
+            multiparts = list(self.backend.get_all_multiparts(bucket_name).values())
             template = Template(S3_ALL_MULTIPARTS)
             return 200, headers, template.render(
                 bucket_name=bucket_name,
                 uploads=multiparts)
-
-        bucket = self.backend.get_bucket(bucket_name)
-        if bucket:
-            prefix = querystring.get('prefix', [None])[0]
+        elif 'versioning' in querystring:
+            versioning = self.backend.get_bucket_versioning(bucket_name)
+            template = Template(S3_BUCKET_GET_VERSIONING)
+            return 200, headers, template.render(status=versioning)
+        elif 'versions' in querystring:
             delimiter = querystring.get('delimiter', [None])[0]
-            result_keys, result_folders = self.backend.prefix_query(bucket, prefix, delimiter)
-            template = Template(S3_BUCKET_GET_RESPONSE)
-            return template.render(
-                bucket=bucket,
-                prefix=prefix,
+            encoding_type = querystring.get('encoding-type', [None])[0]
+            key_marker = querystring.get('key-marker', [None])[0]
+            max_keys = querystring.get('max-keys', [None])[0]
+            prefix = querystring.get('prefix', [None])[0]
+            version_id_marker = querystring.get('version-id-marker', [None])[0]
+
+            bucket = self.backend.get_bucket(bucket_name)
+            versions = self.backend.get_bucket_versions(
+                bucket_name,
                 delimiter=delimiter,
-                result_keys=result_keys,
-                result_folders=result_folders
+                encoding_type=encoding_type,
+                key_marker=key_marker,
+                max_keys=max_keys,
+                version_id_marker=version_id_marker
             )
-        else:
+            template = Template(S3_BUCKET_GET_VERSIONS)
+            return 200, headers, template.render(
+                key_list=versions,
+                bucket=bucket,
+                prefix='',
+                max_keys='',
+                delimiter='',
+                is_truncated='false',
+            )
+
+        try:
+            bucket = self.backend.get_bucket(bucket_name)
+        except MissingBucket:
             return 404, headers, ""
 
-    def _bucket_response_put(self, bucket_name, headers):
-        try:
-            new_bucket = self.backend.create_bucket(bucket_name)
-        except BucketAlreadyExists:
-            return 409, headers, ""
-        template = Template(S3_BUCKET_CREATE_RESPONSE)
-        return template.render(bucket=new_bucket)
+        prefix = querystring.get('prefix', [None])[0]
+        delimiter = querystring.get('delimiter', [None])[0]
+        result_keys, result_folders = self.backend.prefix_query(bucket, prefix, delimiter)
+        template = Template(S3_BUCKET_GET_RESPONSE)
+        return 200, headers, template.render(
+            bucket=bucket,
+            prefix=prefix,
+            delimiter=delimiter,
+            result_keys=result_keys,
+            result_folders=result_folders
+        )
+
+    def _bucket_response_put(self, request, bucket_name, querystring, headers):
+        if 'versioning' in querystring:
+            ver = re.search('<Status>([A-Za-z]+)</Status>', request.body.decode('utf-8'))
+            if ver:
+                self.backend.set_bucket_versioning(bucket_name, ver.group(1))
+                template = Template(S3_BUCKET_VERSIONING)
+                return template.render(bucket_versioning_status=ver.group(1))
+            else:
+                return 404, headers, ""
+        else:
+            try:
+                new_bucket = self.backend.create_bucket(bucket_name)
+            except BucketAlreadyExists:
+                return 409, headers, ""
+            template = Template(S3_BUCKET_CREATE_RESPONSE)
+            return 200, headers, template.render(bucket=new_bucket)
 
     def _bucket_response_delete(self, bucket_name, headers):
-        removed_bucket = self.backend.delete_bucket(bucket_name)
-        if removed_bucket is None:
+        try:
+            removed_bucket = self.backend.delete_bucket(bucket_name)
+        except MissingBucket:
             # Non-existant bucket
             template = Template(S3_DELETE_NON_EXISTING_BUCKET)
             return 404, headers, template.render(bucket_name=bucket_name)
-        elif removed_bucket:
+
+        if removed_bucket:
             # Bucket exists
             template = Template(S3_DELETE_BUCKET_SUCCESS)
             return 204, headers, template.render(bucket=removed_bucket)
@@ -114,14 +163,17 @@ class ResponseObject(object):
             return 409, headers, template.render(bucket=removed_bucket)
 
     def _bucket_response_post(self, request, bucket_name, headers):
-        #POST to bucket-url should create file from form
+        if request.path == u'/?delete':
+            return self._bucket_response_delete_keys(request, bucket_name, headers)
+
+        # POST to bucket-url should create file from form
         if hasattr(request, 'form'):
-            #Not HTTPretty
+            # Not HTTPretty
             form = request.form
         else:
-            #HTTPretty, build new form object
+            # HTTPretty, build new form object
             form = {}
-            for kv in request.body.split('&'):
+            for kv in request.body.decode('utf-8').split('&'):
                 k, v = kv.split('=')
                 form[k] = v
 
@@ -133,7 +185,7 @@ class ResponseObject(object):
 
         new_key = self.backend.set_key(bucket_name, key, f)
 
-        #Metadata
+        # Metadata
         meta_regex = re.compile('^x-amz-meta-([a-zA-Z0-9\-_]+)$', flags=re.IGNORECASE)
 
         for form_id in form:
@@ -144,9 +196,30 @@ class ResponseObject(object):
                 new_key.set_metadata(meta_key, metadata)
         return 200, headers, ""
 
+    def _bucket_response_delete_keys(self, request, bucket_name, headers):
+        template = Template(S3_DELETE_KEYS_RESPONSE)
+
+        keys = minidom.parseString(request.body.decode('utf-8')).getElementsByTagName('Key')
+        deleted_names = []
+        error_names = []
+
+        for k in keys:
+            try:
+                key_name = k.firstChild.nodeValue
+                self.backend.delete_key(bucket_name, key_name)
+                deleted_names.append(key_name)
+            except KeyError:
+                error_names.append(key_name)
+
+        return 200, headers, template.render(deleted=deleted_names, delete_errors=error_names)
+
     def key_response(self, request, full_url, headers):
-        response = self._key_response(request, full_url, headers)
-        if isinstance(response, basestring):
+        try:
+            response = self._key_response(request, full_url, headers)
+        except MissingBucket:
+            return 404, headers, ""
+
+        if isinstance(response, six.string_types):
             return 200, headers, response
         else:
             status_code, headers, response_content = response
@@ -156,8 +229,8 @@ class ResponseObject(object):
         meta_regex = re.compile('^x-amz-meta-([a-zA-Z0-9\-_]+)$', flags=re.IGNORECASE)
         if replace is True:
             key.clear_metadata()
-        for header in request.headers:
-            if isinstance(header, basestring):
+        for header, value in request.headers.items():
+            if isinstance(header, six.string_types):
                 result = meta_regex.match(header)
                 if result:
                     meta_key = result.group(0).lower()
@@ -182,7 +255,7 @@ class ResponseObject(object):
         if method == 'GET':
             return self._key_response_get(bucket_name, query, key_name, headers)
         elif method == 'PUT':
-            return self._key_response_put(request, body, bucket_name, query, key_name, headers)
+            return self._key_response_put(request, parsed_url, body, bucket_name, query, key_name, headers)
         elif method == 'HEAD':
             return self._key_response_head(bucket_name, key_name, headers)
         elif method == 'DELETE':
@@ -204,14 +277,16 @@ class ResponseObject(object):
                 count=len(parts),
                 parts=parts
             )
-        key = self.backend.get_key(bucket_name, key_name)
+        version_id = query.get('versionId', [None])[0]
+        key = self.backend.get_key(
+            bucket_name, key_name, version_id=version_id)
         if key:
             headers.update(key.metadata)
             return 200, headers, key.value
         else:
             return 404, headers, ""
 
-    def _key_response_put(self, request, body, bucket_name, query, key_name, headers):
+    def _key_response_put(self, request, parsed_url, body, bucket_name, query, key_name, headers):
         if 'uploadId' in query and 'partNumber' in query:
             upload_id = query['uploadId'][0]
             part_number = int(query['partNumber'][0])
@@ -232,6 +307,10 @@ class ResponseObject(object):
 
         storage_class = request.headers.get('x-amz-storage-class', 'STANDARD')
 
+        if parsed_url.query == 'acl':
+            # We don't implement ACL yet, so just return
+            return 200, headers, ""
+
         if 'x-amz-copy-source' in request.headers:
             # Copy key
             src_bucket, src_key = request.headers.get("x-amz-copy-source").split("/", 1)
@@ -281,7 +360,7 @@ class ResponseObject(object):
         return 204, headers, template.render(bucket=removed_key)
 
     def _key_response_post(self, body, parsed_url, bucket_name, query, key_name, headers):
-        if body == '' and parsed_url.query == 'uploads':
+        if body == b'' and parsed_url.query == 'uploads':
             multipart = self.backend.initiate_multipart(bucket_name, key_name)
             template = Template(S3_MULTIPART_INITIATE_RESPONSE)
             response = template.render(
@@ -391,6 +470,63 @@ S3_DELETE_BUCKET_WITH_ITEMS_ERROR = """<?xml version="1.0" encoding="UTF-8"?>
 <HostId>sdfgdsfgdsfgdfsdsfgdfs</HostId>
 </Error>"""
 
+S3_BUCKET_VERSIONING = """
+<?xml version="1.0" encoding="UTF-8"?>
+<VersioningConfiguration xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
+    <Status>{{ bucket_versioning_status }}</Status>
+</VersioningConfiguration>
+"""
+
+S3_BUCKET_GET_VERSIONING = """
+<?xml version="1.0" encoding="UTF-8"?>
+{% if status is none %}
+    <VersioningConfiguration xmlns="http://s3.amazonaws.com/doc/2006-03-01/"/>
+{% else %}
+    <VersioningConfiguration xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
+    <Status>{{ status }}</Status>
+    </VersioningConfiguration>
+{% endif %}
+"""
+
+S3_BUCKET_GET_VERSIONS = """<?xml version="1.0" encoding="UTF-8"?>
+<ListVersionsResult xmlns="http://s3.amazonaws.com/doc/2006-03-01">
+    <Name>{{ bucket.name }}</Name>
+    <Prefix>{{ prefix }}</Prefix>
+    <KeyMarker>{{ key_marker }}</KeyMarker>
+    <MaxKeys>{{ max_keys }}</MaxKeys>
+    <IsTruncated>{{ is_truncated }}</IsTruncated>
+    {% for key in key_list %}
+    <Version>
+        <Key>{{ key.name }}</Key>
+        <VersionId>{{ key._version_id }}</VersionId>
+        <IsLatest>false</IsLatest>
+        <LastModified>{{ key.last_modified_ISO8601 }}</LastModified>
+        <ETag>{{ key.etag }}</ETag>
+        <Size>{{ key.size }}</Size>
+        <StorageClass>{{ key.storage_class }}</StorageClass>
+        <Owner>
+            <ID>75aa57f09aa0c8caeab4f8c24e99d10f8e7faeebf76c078efc7c6caea54ba06a</ID>
+            <DisplayName>webfile</DisplayName>
+        </Owner>
+    </Version>
+    {% endfor %}
+</ListVersionsResult>
+"""
+
+S3_DELETE_KEYS_RESPONSE = """<?xml version="1.0" encoding="UTF-8"?>
+<DeleteResult xmlns="http://s3.amazonaws.com/doc/2006-03-01">
+{% for k in deleted %}
+<Deleted>
+<Key>{{k}}</Key>
+</Deleted>
+{% endfor %}
+{% for k in delete_errors %}
+<Error>
+<Key>{{k}}</Key>
+</Error>
+{% endfor %}
+</DeleteResult>"""
+
 S3_DELETE_OBJECT_SUCCESS = """<DeleteObjectResponse xmlns="http://s3.amazonaws.com/doc/2006-03-01">
   <DeleteObjectResponse>
     <Code>200</Code>
