@@ -1,7 +1,8 @@
+from __future__ import unicode_literals
 from .responses import S3ResponseInstance
 
 url_bases = [
-    "https?://(?P<bucket_name>[a-zA-Z0-9\-_.]*)\.?s3.amazonaws.com"
+    "https?://(?P<bucket_name>[a-zA-Z0-9\-_.]*)\.?s3(.*).amazonaws.com"
 ]
 
 url_paths = {
