@@ -1,8 +1,9 @@
-import urlparse
+from __future__ import unicode_literals
+from six.moves.urllib.parse import urlparse
 
 
 def bucket_name_from_url(url):
-    pth = urlparse.urlparse(url).path.lstrip("/")
+    pth = urlparse(url).path.lstrip("/")
 
     l = pth.lstrip("/").split("/")
     if len(l) == 0 or l[0] == "":
