@@ -1,25 +1,41 @@
+from __future__ import unicode_literals
 from jinja2 import Template
 
 from moto.core.responses import BaseResponse
 from moto.core.utils import camelcase_to_underscores
-from .models import sqs_backend
+from .utils import parse_message_attributes
+from .models import sqs_backends
+from .exceptions import (
+    MessageAttributesInvalid,
+    MessageNotInflight,
+    ReceiptHandleIsInvalid
+)
+
+MAXIMUM_VISIBILTY_TIMEOUT = 43200
+SQS_REGION_REGEX = r'://(.+?)\.queue\.amazonaws\.com'
 
 
 class QueuesResponse(BaseResponse):
 
+    region_regex = SQS_REGION_REGEX
+
+    @property
+    def sqs_backend(self):
+        return sqs_backends[self.region]
+
     def create_queue(self):
         visibility_timeout = None
         if 'Attribute.1.Name' in self.querystring and self.querystring.get('Attribute.1.Name')[0] == 'VisibilityTimeout':
             visibility_timeout = self.querystring.get("Attribute.1.Value")[0]
 
         queue_name = self.querystring.get("QueueName")[0]
-        queue = sqs_backend.create_queue(queue_name, visibility_timeout=visibility_timeout)
+        queue = self.sqs_backend.create_queue(queue_name, visibility_timeout=visibility_timeout)
         template = Template(CREATE_QUEUE_RESPONSE)
         return template.render(queue=queue)
 
     def get_queue_url(self):
         queue_name = self.querystring.get("QueueName")[0]
-        queue = sqs_backend.get_queue(queue_name)
+        queue = self.sqs_backend.get_queue(queue_name)
         if queue:
             template = Template(GET_QUEUE_URL_RESPONSE)
             return template.render(queue=queue)
@@ -28,15 +44,44 @@ class QueuesResponse(BaseResponse):
 
     def list_queues(self):
         queue_name_prefix = self.querystring.get("QueueNamePrefix", [None])[0]
-        queues = sqs_backend.list_queues(queue_name_prefix)
+        queues = self.sqs_backend.list_queues(queue_name_prefix)
         template = Template(LIST_QUEUES_RESPONSE)
         return template.render(queues=queues)
 
 
 class QueueResponse(BaseResponse):
+
+    region_regex = SQS_REGION_REGEX
+
+    @property
+    def sqs_backend(self):
+        return sqs_backends[self.region]
+
+    def change_message_visibility(self):
+        queue_name = self.path.split("/")[-1]
+        receipt_handle = self.querystring.get("ReceiptHandle")[0]
+        visibility_timeout = int(self.querystring.get("VisibilityTimeout")[0])
+
+        if visibility_timeout > MAXIMUM_VISIBILTY_TIMEOUT:
+            return "Invalid request, maximum visibility timeout is {0}".format(
+                MAXIMUM_VISIBILTY_TIMEOUT
+            ), dict(status=400)
+
+        try:
+            self.sqs_backend.change_message_visibility(
+                queue_name=queue_name,
+                receipt_handle=receipt_handle,
+                visibility_timeout=visibility_timeout
+            )
+        except (ReceiptHandleIsInvalid, MessageNotInflight) as e:
+            return "Invalid request: {0}".format(e.description), dict(status=e.status_code)
+
+        template = Template(CHANGE_MESSAGE_VISIBILITY_RESPONSE)
+        return template.render()
+
     def get_queue_attributes(self):
         queue_name = self.path.split("/")[-1]
-        queue = sqs_backend.get_queue(queue_name)
+        queue = self.sqs_backend.get_queue(queue_name)
         template = Template(GET_QUEUE_ATTRIBUTES_RESPONSE)
         return template.render(queue=queue)
 
@@ -44,12 +89,12 @@ class QueueResponse(BaseResponse):
         queue_name = self.path.split("/")[-1]
         key = camelcase_to_underscores(self.querystring.get('Attribute.Name')[0])
         value = self.querystring.get('Attribute.Value')[0]
-        sqs_backend.set_queue_attribute(queue_name, key, value)
+        self.sqs_backend.set_queue_attribute(queue_name, key, value)
         return SET_QUEUE_ATTRIBUTE_RESPONSE
 
     def delete_queue(self):
         queue_name = self.path.split("/")[-1]
-        queue = sqs_backend.delete_queue(queue_name)
+        queue = self.sqs_backend.delete_queue(queue_name)
         if not queue:
             return "A queue with name {0} does not exist".format(queue_name), dict(status=404)
         template = Template(DELETE_QUEUE_RESPONSE)
@@ -57,10 +102,27 @@ class QueueResponse(BaseResponse):
 
     def send_message(self):
         message = self.querystring.get("MessageBody")[0]
+        delay_seconds = self.querystring.get('DelaySeconds')
+
+        if delay_seconds:
+            delay_seconds = int(delay_seconds[0])
+        else:
+            delay_seconds = 0
+
+        try:
+            message_attributes = parse_message_attributes(self.querystring)
+        except MessageAttributesInvalid as e:
+            return e.description, dict(status=e.status_code)
+
         queue_name = self.path.split("/")[-1]
-        message = sqs_backend.send_message(queue_name, message)
+        message = self.sqs_backend.send_message(
+            queue_name,
+            message,
+            message_attributes=message_attributes,
+            delay_seconds=delay_seconds
+        )
         template = Template(SEND_MESSAGE_RESPONSE)
-        return template.render(message=message)
+        return template.render(message=message, message_attributes=message_attributes)
 
     def send_message_batch(self):
         """
@@ -89,8 +151,14 @@ class QueueResponse(BaseResponse):
             message_user_id = self.querystring.get(message_user_id_key)[0]
             delay_key = 'SendMessageBatchRequestEntry.{0}.DelaySeconds'.format(index)
             delay_seconds = self.querystring.get(delay_key, [None])[0]
-            message = sqs_backend.send_message(queue_name, message_body[0], delay_seconds=delay_seconds)
+            message = self.sqs_backend.send_message(queue_name, message_body[0], delay_seconds=delay_seconds)
             message.user_id = message_user_id
+
+            message_attributes = parse_message_attributes(self.querystring, base='SendMessageBatchRequestEntry.{0}.'.format(index), value_namespace='')
+            if type(message_attributes) == tuple:
+                return message_attributes[0], message_attributes[1]
+            message.message_attributes = message_attributes
+
             messages.append(message)
 
         template = Template(SEND_MESSAGE_BATCH_RESPONSE)
@@ -99,7 +167,7 @@ class QueueResponse(BaseResponse):
     def delete_message(self):
         queue_name = self.path.split("/")[-1]
         receipt_handle = self.querystring.get("ReceiptHandle")[0]
-        sqs_backend.delete_message(queue_name, receipt_handle)
+        self.sqs_backend.delete_message(queue_name, receipt_handle)
         template = Template(DELETE_MESSAGE_RESPONSE)
         return template.render()
 
@@ -124,7 +192,7 @@ class QueueResponse(BaseResponse):
                 # Found all messages
                 break
 
-            sqs_backend.delete_message(queue_name, receipt_handle[0])
+            self.sqs_backend.delete_message(queue_name, receipt_handle[0])
 
             message_user_id_key = 'DeleteMessageBatchRequestEntry.{0}.Id'.format(index)
             message_user_id = self.querystring.get(message_user_id_key)[0]
@@ -136,9 +204,10 @@ class QueueResponse(BaseResponse):
     def receive_message(self):
         queue_name = self.path.split("/")[-1]
         message_count = int(self.querystring.get("MaxNumberOfMessages")[0])
-        messages = sqs_backend.receive_messages(queue_name, message_count)
+        messages = self.sqs_backend.receive_messages(queue_name, message_count)
         template = Template(RECEIVE_MESSAGE_RESPONSE)
-        return template.render(messages=messages)
+        output = template.render(messages=messages)
+        return output
 
 
 CREATE_QUEUE_RESPONSE = """<CreateQueueResponse>
@@ -211,6 +280,9 @@ SEND_MESSAGE_RESPONSE = """<SendMessageResponse>
         <MD5OfMessageBody>
             {{ message.md5 }}
         </MD5OfMessageBody>
+        {% if message.message_attributes.items()|count > 0 %}
+          <MD5OfMessageAttributes>324758f82d026ac6ec5b31a3b192d1e3</MD5OfMessageAttributes>
+        {% endif %}
         <MessageId>
             {{ message.id }}
         </MessageId>
@@ -226,18 +298,42 @@ RECEIVE_MESSAGE_RESPONSE = """<ReceiveMessageResponse>
   <ReceiveMessageResult>
     {% for message in messages %}
         <Message>
-          <MessageId>
-            {{ message.id }}
-          </MessageId>
-          <ReceiptHandle>
-            MbZj6wDWli+JvwwJaBV+3dcjk2YW2vA3+STFFljTM8tJJg6HRG6PYSasuWXPJB+Cw
-            Lj1FjgXUv1uSj1gUPAWV66FU/WeR4mq2OKpEGYWbnLmpRCJVAyeMjeU5ZBdtcQ+QE
-            auMZc8ZRv37sIW2iJKq3M9MFx1YvV11A2x/KSbkJ0=
-          </ReceiptHandle>
-          <MD5OfBody>
-            {{ message.md5 }}
-          </MD5OfBody>
+          <MessageId>{{ message.id }}</MessageId>
+          <ReceiptHandle>{{ message.receipt_handle }}</ReceiptHandle>
+          <MD5OfBody>{{ message.md5 }}</MD5OfBody>
           <Body>{{ message.body }}</Body>
+          <Attribute>
+            <Name>SenderId</Name>
+            <Value>{{ message.sender_id }}</Value>
+          </Attribute>
+          <Attribute>
+            <Name>SentTimestamp</Name>
+            <Value>{{ message.sent_timestamp }}</Value>
+          </Attribute>
+          <Attribute>
+            <Name>ApproximateReceiveCount</Name>
+            <Value>{{ message.approximate_receive_count }}</Value>
+          </Attribute>
+          <Attribute>
+            <Name>ApproximateFirstReceiveTimestamp</Name>
+            <Value>{{ message.approximate_first_receive_timestamp }}</Value>
+          </Attribute>
+          {% if message.message_attributes.items()|count > 0 %}
+            <MD5OfMessageAttributes>324758f82d026ac6ec5b31a3b192d1e3</MD5OfMessageAttributes>
+          {% endif %}
+          {% for name, value in message.message_attributes.items() %}
+            <MessageAttribute>
+              <Name>{{ name }}</Name>
+              <Value>
+                <DataType>{{ value.data_type }}</DataType>
+                {% if 'Binary' in value.data_type %}
+                <BinaryValue>{{ value.binary_value }}</BinaryValue>
+                {% else %}
+                <StringValue>{{ value.string_value }}</StringValue>
+                {% endif %}
+              </Value>
+            </MessageAttribute>
+          {% endfor %}
         </Message>
     {% endfor %}
   </ReceiveMessageResult>
@@ -255,6 +351,9 @@ SEND_MESSAGE_BATCH_RESPONSE = """<SendMessageBatchResponse>
             <Id>{{ message.user_id }}</Id>
             <MessageId>{{ message.id }}</MessageId>
             <MD5OfMessageBody>{{ message.md5 }}</MD5OfMessageBody>
+            {% if message.message_attributes.items()|count > 0 %}
+              <MD5OfMessageAttributes>324758f82d026ac6ec5b31a3b192d1e3</MD5OfMessageAttributes>
+            {% endif %}
         </SendMessageBatchResultEntry>
     {% endfor %}
 </SendMessageBatchResult>
@@ -283,3 +382,11 @@ DELETE_MESSAGE_BATCH_RESPONSE = """<DeleteMessageBatchResponse>
         <RequestId>d6f86b7a-74d1-4439-b43f-196a1e29cd85</RequestId>
     </ResponseMetadata>
 </DeleteMessageBatchResponse>"""
+
+CHANGE_MESSAGE_VISIBILITY_RESPONSE = """<ChangeMessageVisibilityResponse>
+    <ResponseMetadata>
+        <RequestId>
+            6a7a282a-d013-4a59-aba9-335b0fa48bed
+        </RequestId>
+    </ResponseMetadata>
+</ChangeMessageVisibilityResponse>"""
